"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  chunk: () => chunk,
  getCollectionCount: () => getCollectionCount,
  getDocument: () => getDocument,
  getDocumentData: () => getDocumentData,
  getDocumentDataMaybe: () => getDocumentDataMaybe,
  getDocumentInTransaction: () => getDocumentInTransaction,
  getDocumentInTransactionMaybe: () => getDocumentInTransactionMaybe,
  getDocumentMaybe: () => getDocumentMaybe,
  getErrorMessage: () => getErrorMessage,
  getSpecificDocument: () => getSpecificDocument,
  getSpecificDocumentData: () => getSpecificDocumentData,
  getSpecificDocumentFromTransaction: () => getSpecificDocumentFromTransaction,
  invariant: () => invariant,
  isDefined: () => isDefined,
  isEmpty: () => isEmpty,
  isFilled: () => isFilled,
  isPresent: () => isPresent,
  makeDocument: () => makeDocument,
  makeMutableDocument: () => makeMutableDocument,
  makeMutableDocumentInTransaction: () => makeMutableDocumentInTransaction,
  useCollection: () => useCollection,
  useCollectionCountOnce: () => useCollectionCountOnce,
  useCollectionMaybe: () => useCollectionMaybe,
  useCollectionOnce: () => useCollectionOnce,
  useDocument: () => useDocument,
  useDocumentData: () => useDocumentData,
  useDocumentDataMaybe: () => useDocumentDataMaybe,
  useDocumentDataOnce: () => useDocumentDataOnce,
  useDocumentMaybe: () => useDocumentMaybe,
  useDocumentOnce: () => useDocumentOnce,
  useSpecificDocument: () => useSpecificDocument,
  useSpecificDocumentData: () => useSpecificDocumentData,
  useSpecificDocumentDataMaybe: () => useSpecificDocumentDataMaybe,
  useSpecificDocumentDataOnce: () => useSpecificDocumentDataOnce,
  useSpecificDocumentMaybe: () => useSpecificDocumentMaybe,
  useSpecificDocumentOnce: () => useSpecificDocumentOnce
});
module.exports = __toCommonJS(src_exports);

// src/get-collection-count.ts
var import_firestore = require("@react-native-firebase/firestore");
async function getCollectionCount(collectionRef, ...queryConstraints) {
  const snapshot = await (0, import_firestore.getCountFromServer)(
    (0, import_firestore.query)(collectionRef, ...queryConstraints)
  );
  return snapshot.data().count;
}

// src/get-document.ts
var import_firestore3 = require("@react-native-firebase/firestore");

// src/utils/invariant.ts
function invariant(condition, msg) {
  if (!condition) {
    throw new Error(msg);
  }
}

// src/utils/is-empty.ts
function isEmpty(array) {
  return array.length === 0;
}

// src/utils/chunk.ts
function chunk(array, size) {
  invariant(size > 0, "Size must be greater than 0");
  if (isEmpty(array)) {
    return [];
  }
  const length = array.length;
  let index = 0;
  let resIndex = 0;
  const result = Array.from({ length: Math.ceil(length / size) });
  while (index < length) {
    result[resIndex++] = array.slice(index, index += size);
  }
  return result;
}

// src/utils/get-error-message.ts
function getErrorMessage(error) {
  return toErrorWithMessage(error).message;
}
function isErrorWithMessage(error) {
  return typeof error === "object" && error !== null && "message" in error && typeof error.message === "string";
}
function toErrorWithMessage(maybeError) {
  if (isErrorWithMessage(maybeError)) return maybeError;
  try {
    return new Error(JSON.stringify(maybeError));
  } catch (e) {
    return new Error(String(maybeError));
  }
}

// src/utils/is-present.ts
function isPresent(t) {
  return t !== void 0 && t !== null;
}
function isDefined(t) {
  return t !== void 0;
}
function isFilled(t) {
  return t !== null;
}

// src/make-mutable-document.ts
var import_firestore2 = require("@react-native-firebase/firestore");
function makeMutableDocument(doc3) {
  return {
    id: doc3.id,
    data: doc3.data(),
    ref: doc3.ref,
    update: (data) => (0, import_firestore2.updateDoc)(doc3.ref, data),
    updateWithPartial: (data) => (0, import_firestore2.updateDoc)(doc3.ref, data),
    delete: () => (0, import_firestore2.deleteDoc)(doc3.ref)
  };
}
function makeMutableDocumentInTransaction(doc3, tx) {
  return {
    id: doc3.id,
    data: doc3.data(),
    ref: doc3.ref,
    update: (data) => tx.update(doc3.ref, data),
    updateWithPartial: (data) => tx.update(doc3.ref, data),
    delete: () => tx.delete(doc3.ref)
  };
}

// src/get-document.ts
async function getDocument(collectionRef, documentId) {
  const snapshot = await (0, import_firestore3.getDoc)((0, import_firestore3.doc)(collectionRef, documentId));
  invariant(
    snapshot.exists,
    `No document available at ${collectionRef.path}/${documentId}`
  );
  return makeMutableDocument(snapshot);
}
async function getDocumentMaybe(collectionRef, documentId) {
  const snapshot = await (0, import_firestore3.getDoc)((0, import_firestore3.doc)(collectionRef, documentId));
  if (!snapshot.exists) return;
  return makeMutableDocument(snapshot);
}
async function getDocumentData(collectionRef, documentId) {
  const docSnap = await (0, import_firestore3.getDoc)((0, import_firestore3.doc)(collectionRef, documentId));
  invariant(
    docSnap.exists,
    `No document available at ${collectionRef.path}/${documentId}`
  );
  return docSnap.data();
}
async function getDocumentDataMaybe(collectionRef, documentId) {
  const snapshot = await (0, import_firestore3.getDoc)((0, import_firestore3.doc)(collectionRef, documentId));
  if (!snapshot.exists) return;
  return snapshot.data();
}
async function getDocumentInTransaction(transaction, collectionRef, documentId) {
  const snapshot = await transaction.get((0, import_firestore3.doc)(collectionRef, documentId));
  invariant(
    snapshot.exists,
    `No document available at ${collectionRef.path}/${documentId}`
  );
  return makeMutableDocument(snapshot);
}
async function getDocumentInTransactionMaybe(transaction, collectionRef, documentId) {
  const snapshot = await transaction.get((0, import_firestore3.doc)(collectionRef, documentId));
  if (!snapshot.exists) {
    return;
  }
  return makeMutableDocument(snapshot);
}

// src/get-specific-document.ts
var import_firestore4 = require("@react-native-firebase/firestore");

// src/make-document.ts
function makeDocument(doc3) {
  return {
    id: doc3.id,
    data: doc3.data()
  };
}

// src/get-specific-document.ts
async function getSpecificDocument(documentRef) {
  const snapshot = await (0, import_firestore4.getDoc)(documentRef);
  invariant(snapshot.exists, `No document available at ${documentRef.path}`);
  return makeMutableDocument(snapshot);
}
async function getSpecificDocumentData(documentRef) {
  const docSnap = await (0, import_firestore4.getDoc)(documentRef);
  invariant(docSnap.exists, `No document available at ${documentRef.path}`);
  return docSnap.data();
}
async function getSpecificDocumentFromTransaction(transaction, documentRef) {
  const snapshot = await transaction.get(documentRef);
  invariant(snapshot.exists, `No document available at ${documentRef.path}`);
  return makeDocument(snapshot);
}

// src/use-collection.ts
var import_firestore7 = require("@react-native-firebase/firestore");
var import_react6 = require("react");

// src/fork/useCollection.ts
var import_firestore5 = require("@react-native-firebase/firestore");
var import_react4 = require("react");

// src/fork/helpers/isEqualQuery.ts
var isEqualQuery = (v1, v2) => {
  if (!v1 || !v2) {
    return v1 === v2;
  }
  return v1.isEqual(v2);
};

// src/fork/helpers/isEqualRef.ts
var isRefEqual = (v1, v2) => {
  return (v1 == null ? void 0 : v1.path) === (v2 == null ? void 0 : v2.path);
};

// src/fork/helpers/refHooks.ts
var import_react = require("react");
var useComparatorRef = (value, isEqual, onChange) => {
  const ref = (0, import_react.useRef)(value);
  (0, import_react.useEffect)(() => {
    if (!isEqual(value, ref.current)) {
      ref.current = value;
      if (onChange) {
        onChange();
      }
    }
  });
  return ref;
};

// src/fork/helpers/useIsFirestoreQueryEqual.ts
var useIsFirestoreQueryEqual = (value, onChange) => {
  return useComparatorRef(value, isEqualQuery, onChange);
};

// src/fork/helpers/useIsFirestoreRefEqual.ts
var useIsFirestoreRefEqual = (value, onChange) => {
  return useComparatorRef(value, isRefEqual, onChange);
};

// src/fork/helpers/useIsMounted.ts
var import_react2 = require("react");
var useIsMounted = () => {
  const [isMounted, setIsMounted] = (0, import_react2.useState)(true);
  (0, import_react2.useEffect)(() => {
    return () => {
      setIsMounted(false);
    };
  }, []);
  return isMounted;
};

// src/fork/helpers/useLoadingValue.ts
var import_react3 = require("react");
var defaultState = (defaultValue) => {
  return {
    loading: defaultValue === void 0 || defaultValue === null,
    value: defaultValue
  };
};
var reducer = () => (state, action) => {
  switch (action.type) {
    case "error":
      return {
        ...state,
        error: action.error,
        loading: false,
        value: void 0
      };
    case "reset":
      return defaultState(action.defaultValue);
    case "value":
      return {
        ...state,
        error: void 0,
        loading: false,
        value: action.value
      };
    default:
      return state;
  }
};
function useLoadingValue(getDefaultValue) {
  const defaultValue = getDefaultValue ? getDefaultValue() : void 0;
  const [state, dispatch] = (0, import_react3.useReducer)(
    reducer(),
    defaultState(defaultValue)
  );
  const reset = (0, import_react3.useCallback)(() => {
    const defaultValue2 = getDefaultValue ? getDefaultValue() : void 0;
    dispatch({ type: "reset", defaultValue: defaultValue2 });
  }, [getDefaultValue]);
  const setError = (0, import_react3.useCallback)((error) => {
    dispatch({ type: "error", error });
  }, []);
  const setValue = (0, import_react3.useCallback)((value) => {
    dispatch({ type: "value", value });
  }, []);
  return (0, import_react3.useMemo)(
    () => ({
      error: state.error,
      loading: state.loading,
      reset,
      setError,
      setValue,
      value: state.value
    }),
    [state.error, state.loading, reset, setError, setValue, state.value]
  );
}

// src/fork/useCollection.ts
function useCollection_fork(query4, options) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const ref = useIsFirestoreQueryEqual(query4, reset);
  (0, import_react4.useEffect)(() => {
    if (!ref.current) {
      setValue(void 0);
      return;
    }
    const unsubscribe = (options == null ? void 0 : options.snapshotListenOptions) ? (0, import_firestore5.onSnapshot)(
      ref.current,
      options.snapshotListenOptions,
      setValue,
      setError
    ) : (0, import_firestore5.onSnapshot)(ref.current, setValue, setError);
    return () => {
      unsubscribe();
    };
  }, [ref.current]);
  return [value, loading, error];
}
function useCollectionOnce_fork(query4, options) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const isMounted = useIsMounted();
  const ref = useIsFirestoreQueryEqual(query4, reset);
  const loadData = (0, import_react4.useCallback)(
    async (query5, options2) => {
      if (!query5) {
        setValue(void 0);
        return;
      }
      const get = getDocsFnFromGetOptions(options2 == null ? void 0 : options2.getOptions);
      try {
        const result = await get(query5);
        if (isMounted) {
          setValue(result);
        }
      } catch (error2) {
        if (isMounted) {
          setError(error2);
        }
      }
    },
    []
  );
  const reloadData = (0, import_react4.useCallback)(
    () => loadData(ref.current, options),
    [loadData, ref.current]
  );
  (0, import_react4.useEffect)(() => {
    loadData(ref.current, options).catch(setError);
  }, [ref.current]);
  return [value, loading, error, reloadData];
}
var getDocsFnFromGetOptions = ({ source } = { source: "default" }) => {
  switch (source) {
    default:
    case "default":
      return import_firestore5.getDocs;
    case "cache":
      return import_firestore5.getDocsFromCache;
    case "server":
      return import_firestore5.getDocsFromServer;
  }
};

// src/fork/useDocument.ts
var import_firestore6 = require("@react-native-firebase/firestore");
var import_react5 = require("react");
var useDocument_fork = (docRef, options) => {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const ref = useIsFirestoreRefEqual(docRef, reset);
  (0, import_react5.useEffect)(() => {
    if (!ref.current) {
      setValue(void 0);
      return;
    }
    const unsubscribe = (options == null ? void 0 : options.snapshotListenOptions) ? (0, import_firestore6.onSnapshot)(
      ref.current,
      options.snapshotListenOptions,
      setValue,
      (err) => {
        if (err instanceof Error) {
          setError(err);
        }
      }
    ) : (0, import_firestore6.onSnapshot)(ref.current, setValue, (err) => {
      if (err instanceof Error) {
        setError(err);
      }
    });
    return () => {
      unsubscribe();
    };
  }, [ref.current]);
  return [value, loading, error];
};
function useDocumentOnce_fork(docRef, options) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const isMounted = useIsMounted();
  const ref = useIsFirestoreRefEqual(docRef, reset);
  const loadData = (0, import_react5.useCallback)(
    async (reference, options2) => {
      if (!reference) {
        setValue(void 0);
        return;
      }
      const get = getDocFnFromGetOptions(options2 == null ? void 0 : options2.getOptions);
      try {
        const result = await get(reference);
        if (isMounted) {
          setValue(result);
        }
      } catch (err) {
        if (isMounted) {
          if (err instanceof Error) {
            setError(err);
          }
        }
      }
    },
    []
  );
  const reloadData = (0, import_react5.useCallback)(
    () => loadData(ref.current, options),
    [loadData, ref.current]
  );
  (0, import_react5.useEffect)(() => {
    if (!ref.current) {
      setValue(void 0);
      return;
    }
    loadData(ref.current, options).catch((err) => {
      if (err instanceof Error) {
        setError(err);
      }
    });
  }, [ref.current]);
  return [value, loading, error, reloadData];
}
var getDocFnFromGetOptions = ({ source } = { source: "default" }) => {
  switch (source) {
    default:
    case "default":
      return import_firestore6.getDoc;
    case "cache":
      return import_firestore6.getDocFromCache;
    case "server":
      return import_firestore6.getDocFromServer;
  }
};

// src/use-collection.ts
function useCollection(collectionRef, ...queryConstraints) {
  const hasNoConstraints = queryConstraints.length === 0;
  const _query = hasNoConstraints ? (0, import_firestore7.query)(collectionRef, (0, import_firestore7.limit)(500)) : (0, import_firestore7.query)(collectionRef, ...queryConstraints.filter(isDefined));
  const [snapshot, , error] = useCollection_fork(_query);
  if (error) {
    throw new Error(
      `Failed to execute query on ${collectionRef.path}: ${getErrorMessage(error)}`
    );
  }
  const docs = (0, import_react6.useMemo)(() => {
    if (!snapshot) {
      return void 0;
    }
    return snapshot.docs.map((doc3) => makeMutableDocument(doc3));
  }, [snapshot]);
  return docs ? [docs, false] : [void 0, true];
}
function useCollectionOnce(collectionRef, ...queryConstraints) {
  const hasNoConstraints = queryConstraints.length === 0;
  const _query = hasNoConstraints ? (0, import_firestore7.query)(collectionRef, (0, import_firestore7.limit)(500)) : (0, import_firestore7.query)(collectionRef, ...queryConstraints.filter(isDefined));
  const [snapshot, , error] = useCollectionOnce_fork(_query);
  if (error) {
    throw new Error(
      `Failed to execute query on ${collectionRef.path}: ${getErrorMessage(error)}`
    );
  }
  const docs = (0, import_react6.useMemo)(() => {
    if (!snapshot) {
      return void 0;
    }
    return snapshot.docs.map((doc3) => makeMutableDocument(doc3));
  }, [snapshot]);
  return docs ? [docs, false] : [void 0, true];
}
function useCollectionMaybe(collectionRef, ...queryConstraints) {
  const hasNoConstraints = queryConstraints.length === 0;
  const _query = hasNoConstraints ? (0, import_firestore7.query)(collectionRef, (0, import_firestore7.limit)(500)) : (0, import_firestore7.query)(collectionRef, ...queryConstraints.filter(isDefined));
  const [snapshot, isLoading] = useCollection_fork(_query);
  const docs = (0, import_react6.useMemo)(() => {
    if (!snapshot) {
      return void 0;
    }
    return snapshot.docs.map((doc3) => makeMutableDocument(doc3));
  }, [snapshot]);
  return [docs, isLoading];
}

// src/use-collection-count.ts
var import_react7 = require("react");
var import_firestore8 = require("@react-native-firebase/firestore");
var import_react8 = require("react");
function useCollectionCount(query4) {
  const { error, loading, reset, setError, setValue, value } = useLoadingValue();
  const isMounted = useIsMounted();
  const ref = useIsFirestoreQueryEqual(query4, reset);
  const loadData = (0, import_react7.useCallback)(async (query5) => {
    if (!query5) {
      setValue(void 0);
      return;
    }
    try {
      const result = await (0, import_firestore8.getCountFromServer)(query5);
      if (isMounted) {
        setValue(result.data().count);
      }
    } catch (error2) {
      if (isMounted) {
        setError(error2);
      }
    }
  }, []);
  const reloadData = (0, import_react7.useCallback)(
    () => loadData(ref.current),
    [loadData, ref.current]
  );
  (0, import_react8.useEffect)(() => {
    loadData(ref.current).catch(setError);
  }, [ref.current]);
  return [value, loading, error, reloadData];
}
function useCollectionCountOnce(collectionRef, ...queryConstraints) {
  const [value, , error] = useCollectionCount(
    (0, import_firestore8.query)(collectionRef, ...queryConstraints.filter(isDefined))
  );
  if (error) {
    throw new Error(
      `Failed to execute query on ${collectionRef.path}: ${getErrorMessage(error)}`
    );
  }
  return value !== void 0 ? [value, false] : [void 0, true];
}

// src/use-document.ts
var import_firestore9 = require("@react-native-firebase/firestore");
var import_react9 = require("react");
function useDocument(collectionRef, documentId) {
  const ref = documentId ? (0, import_firestore9.doc)(collectionRef, documentId) : void 0;
  const [snapshot, , error] = useDocument_fork(ref);
  if (error) {
    throw error;
  }
  const document = (0, import_react9.useMemo)(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useDocumentMaybe(collectionRef, documentId) {
  const ref = documentId ? (0, import_firestore9.doc)(collectionRef, documentId) : void 0;
  const [snapshot, isLoading] = useDocument_fork(ref);
  const document = (0, import_react9.useMemo)(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return [document, isLoading];
}
function useDocumentData(collectionRef, documentId) {
  const [document, isLoading] = useDocument(collectionRef, documentId);
  return isLoading ? [void 0, true] : [document.data, false];
}
function useDocumentDataMaybe(collectionRef, documentId) {
  const [document, isLoading] = useDocumentMaybe(collectionRef, documentId);
  return [document == null ? void 0 : document.data, isLoading];
}
function useDocumentOnce(collectionRef, documentId) {
  const ref = documentId ? (0, import_firestore9.doc)(collectionRef, documentId) : void 0;
  const [snapshot, , error] = useDocumentOnce_fork(ref);
  if (error) {
    throw error;
  }
  const document = (0, import_react9.useMemo)(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useDocumentDataOnce(collectionRef, documentId) {
  const [document, isLoading] = useDocumentOnce(collectionRef, documentId);
  return isLoading ? [void 0, true] : [document.data, false];
}

// src/use-specific-document.ts
var import_react10 = require("react");
function useSpecificDocument(documentRef) {
  const [snapshot, , error] = useDocument_fork(documentRef);
  if (error) {
    throw error;
  }
  const document = (0, import_react10.useMemo)(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useSpecificDocumentMaybe(documentRef) {
  const [snapshot, isLoading] = useDocument_fork(documentRef);
  const document = (0, import_react10.useMemo)(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return [document, isLoading];
}
function useSpecificDocumentData(documentRef) {
  const [document, isLoading] = useSpecificDocument(documentRef);
  return isLoading ? [void 0, true] : [document.data, false];
}
function useSpecificDocumentDataMaybe(documentRef) {
  const [document, isLoading] = useSpecificDocumentMaybe(documentRef);
  return [document == null ? void 0 : document.data, isLoading];
}
function useSpecificDocumentOnce(documentRef) {
  const [snapshot, , error] = useDocumentOnce_fork(documentRef);
  if (error) {
    throw error;
  }
  const document = (0, import_react10.useMemo)(
    () => (snapshot == null ? void 0 : snapshot.exists) ? makeMutableDocument(snapshot) : void 0,
    [snapshot]
  );
  return document ? [document, false] : [void 0, true];
}
function useSpecificDocumentDataOnce(documentRef) {
  const [document, isLoading] = useSpecificDocumentOnce(documentRef);
  return isLoading ? [void 0, true] : [document.data, false];
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  chunk,
  getCollectionCount,
  getDocument,
  getDocumentData,
  getDocumentDataMaybe,
  getDocumentInTransaction,
  getDocumentInTransactionMaybe,
  getDocumentMaybe,
  getErrorMessage,
  getSpecificDocument,
  getSpecificDocumentData,
  getSpecificDocumentFromTransaction,
  invariant,
  isDefined,
  isEmpty,
  isFilled,
  isPresent,
  makeDocument,
  makeMutableDocument,
  makeMutableDocumentInTransaction,
  useCollection,
  useCollectionCountOnce,
  useCollectionMaybe,
  useCollectionOnce,
  useDocument,
  useDocumentData,
  useDocumentDataMaybe,
  useDocumentDataOnce,
  useDocumentMaybe,
  useDocumentOnce,
  useSpecificDocument,
  useSpecificDocumentData,
  useSpecificDocumentDataMaybe,
  useSpecificDocumentDataOnce,
  useSpecificDocumentMaybe,
  useSpecificDocumentOnce
});
//# sourceMappingURL=index.js.map